@@ -1,37 +1,54 @@
 """All pytest-django fixtures"""
-
-
 import os
 from contextlib import contextmanager
 from functools import partial
+from typing import (
+    Any, Callable, Generator, Iterable, List, Optional, Tuple, Union,
+)
 
 import pytest
 
 from . import live_server_helper
 from .django_compat import is_django_unittest
-from .lazy_django import skip_if_no_django
+from .lazy_django import get_django_version, skip_if_no_django
+
+
+TYPE_CHECKING = False
+if TYPE_CHECKING:
+    from typing import Literal
+
+    import django
+
+    _DjangoDbDatabases = Optional[Union["Literal['__all__']", Iterable[str]]]
+    # transaction, reset_sequences, databases, serialized_rollback
+    _DjangoDb = Tuple[bool, bool, _DjangoDbDatabases, bool]
+
 
 __all__ = [
     "django_db_setup",
     "db",
     "transactional_db",
     "django_db_reset_sequences",
+    "django_db_serialized_rollback",
     "admin_user",
     "django_user_model",
     "django_username_field",
     "client",
+    "async_client",
     "admin_client",
     "rf",
+    "async_rf",
     "settings",
     "live_server",
     "_live_server_helper",
     "django_assert_num_queries",
     "django_assert_max_num_queries",
+    "django_capture_on_commit_callbacks",
 ]
 
 
 @pytest.fixture(scope="session")
-def django_db_modify_db_settings_tox_suffix():
+def django_db_modify_db_settings_tox_suffix() -> None:
     skip_if_no_django()
 
     tox_environment = os.getenv("TOX_PARALLEL_ENV")
@@ -41,7 +58,7 @@ def django_db_modify_db_settings_tox_suffix():
 
 
 @pytest.fixture(scope="session")
-def django_db_modify_db_settings_xdist_suffix(request):
+def django_db_modify_db_settings_xdist_suffix(request) -> None:
     skip_if_no_django()
 
     xdist_suffix = getattr(request.config, "workerinput", {}).get("workerid")
@@ -52,49 +69,51 @@ def django_db_modify_db_settings_xdist_suffix(request):
 
 @pytest.fixture(scope="session")
 def django_db_modify_db_settings_parallel_suffix(
-    django_db_modify_db_settings_tox_suffix,
-    django_db_modify_db_settings_xdist_suffix,
-):
+    django_db_modify_db_settings_tox_suffix: None,
+    django_db_modify_db_settings_xdist_suffix: None,
+) -> None:
     skip_if_no_django()
 
 
 @pytest.fixture(scope="session")
-def django_db_modify_db_settings(django_db_modify_db_settings_parallel_suffix):
+def django_db_modify_db_settings(
+    django_db_modify_db_settings_parallel_suffix: None,
+) -> None:
     skip_if_no_django()
 
 
 @pytest.fixture(scope="session")
-def django_db_use_migrations(request):
+def django_db_use_migrations(request) -> bool:
     return not request.config.getvalue("nomigrations")
 
 
 @pytest.fixture(scope="session")
-def django_db_keepdb(request):
+def django_db_keepdb(request) -> bool:
     return request.config.getvalue("reuse_db")
 
 
 @pytest.fixture(scope="session")
-def django_db_createdb(request):
+def django_db_createdb(request) -> bool:
     return request.config.getvalue("create_db")
 
 
 @pytest.fixture(scope="session")
 def django_db_setup(
     request,
-    django_test_environment,
+    django_test_environment: None,
     django_db_blocker,
-    django_db_use_migrations,
-    django_db_keepdb,
-    django_db_createdb,
-    django_db_modify_db_settings,
-):
+    django_db_use_migrations: bool,
+    django_db_keepdb: bool,
+    django_db_createdb: bool,
+    django_db_modify_db_settings: None,
+) -> None:
     """Top level fixture to ensure test databases are available"""
     from django.test.utils import setup_databases, teardown_databases
 
     setup_databases_args = {}
 
     if not django_db_use_migrations:
-        _disable_native_migrations()
+        _disable_migrations()
 
     if django_db_keepdb and not django_db_createdb:
         setup_databases_args["keepdb"] = True
@@ -106,7 +125,7 @@ def django_db_setup(
             **setup_databases_args
         )
 
-    def teardown_database():
+    def teardown_database() -> None:
         with django_db_blocker.unblock():
             try:
                 teardown_databases(db_cfg, verbosity=request.config.option.verbose)
@@ -121,41 +140,133 @@ def django_db_setup(
         request.addfinalizer(teardown_database)
 
 
-def _django_db_fixture_helper(
-    request, django_db_blocker, transactional=False, reset_sequences=False
-):
+@pytest.fixture()
+def _django_db_helper(
+    request,
+    django_db_setup: None,
+    django_db_blocker,
+) -> None:
+    from django import VERSION
+
     if is_django_unittest(request):
         return
 
-    if not transactional and "live_server" in request.fixturenames:
-        # Do nothing, we get called with transactional=True, too.
-        return
+    marker = request.node.get_closest_marker("django_db")
+    if marker:
+        (
+            transactional,
+            reset_sequences,
+            databases,
+            serialized_rollback,
+        ) = validate_django_db(marker)
+    else:
+        (
+            transactional,
+            reset_sequences,
+            databases,
+            serialized_rollback,
+        ) = False, False, None, False
+
+    transactional = transactional or (
+        "transactional_db" in request.fixturenames
+        or "live_server" in request.fixturenames
+    )
+    reset_sequences = reset_sequences or (
+        "django_db_reset_sequences" in request.fixturenames
+    )
+    serialized_rollback = serialized_rollback or (
+        "django_db_serialized_rollback" in request.fixturenames
+    )
 
     django_db_blocker.unblock()
     request.addfinalizer(django_db_blocker.restore)
 
+    import django.db
+    import django.test
+
     if transactional:
-        from django.test import TransactionTestCase as django_case
-
-        if reset_sequences:
-
-            class ResetSequenceTestCase(django_case):
-                reset_sequences = True
-
-            django_case = ResetSequenceTestCase
+        test_case_class = django.test.TransactionTestCase
     else:
-        from django.test import TestCase as django_case
+        test_case_class = django.test.TestCase
 
-    test_case = django_case(methodName="__init__")
+    _reset_sequences = reset_sequences
+    _serialized_rollback = serialized_rollback
+    _databases = databases
+
+    class PytestDjangoTestCase(test_case_class):  # type: ignore[misc,valid-type]
+        reset_sequences = _reset_sequences
+        serialized_rollback = _serialized_rollback
+        if _databases is not None:
+            databases = _databases
+
+        # For non-transactional tests, skip executing `django.test.TestCase`'s
+        # `setUpClass`/`tearDownClass`, only execute the super class ones.
+        #
+        # `TestCase`'s class setup manages the `setUpTestData`/class-level
+        # transaction functionality. We don't use it; instead we (will) offer
+        # our own alternatives. So it only adds overhead, and does some things
+        # which conflict with our (planned) functionality, particularly, it
+        # closes all database connections in `tearDownClass` which inhibits
+        # wrapping tests in higher-scoped transactions.
+        #
+        # It's possible a new version of Django will add some unrelated
+        # functionality to these methods, in which case skipping them completely
+        # would not be desirable. Let's cross that bridge when we get there...
+        if not transactional:
+            @classmethod
+            def setUpClass(cls) -> None:
+                super(django.test.TestCase, cls).setUpClass()
+                if (3, 2) <= VERSION < (4, 1):
+                    django.db.transaction.Atomic._ensure_durability = False
+
+            @classmethod
+            def tearDownClass(cls) -> None:
+                if (3, 2) <= VERSION < (4, 1):
+                    django.db.transaction.Atomic._ensure_durability = True
+                super(django.test.TestCase, cls).tearDownClass()
+
+    PytestDjangoTestCase.setUpClass()
+    if VERSION >= (4, 0):
+        request.addfinalizer(PytestDjangoTestCase.doClassCleanups)
+    request.addfinalizer(PytestDjangoTestCase.tearDownClass)
+
+    test_case = PytestDjangoTestCase(methodName="__init__")
     test_case._pre_setup()
     request.addfinalizer(test_case._post_teardown)
 
 
-def _disable_native_migrations():
+def validate_django_db(marker) -> "_DjangoDb":
+    """Validate the django_db marker.
+
+    It checks the signature and creates the ``transaction``,
+    ``reset_sequences``, ``databases`` and ``serialized_rollback`` attributes on
+    the marker which will have the correct values.
+
+    Sequence reset and serialized_rollback are only allowed when combined with
+    transaction.
+    """
+
+    def apifun(
+        transaction: bool = False,
+        reset_sequences: bool = False,
+        databases: "_DjangoDbDatabases" = None,
+        serialized_rollback: bool = False,
+    ) -> "_DjangoDb":
+        return transaction, reset_sequences, databases, serialized_rollback
+
+    return apifun(*marker.args, **marker.kwargs)
+
+
+def _disable_migrations() -> None:
     from django.conf import settings
     from django.core.management.commands import migrate
 
-    from .migrations import DisableMigrations
+    class DisableMigrations:
+        def __contains__(self, item: str) -> bool:
+            return True
+
+        def __getitem__(self, item: str) -> None:
+            return None
 
     settings.MIGRATION_MODULES = DisableMigrations()
 
@@ -167,7 +278,7 @@ def _disable_native_migrations():
     migrate.Command = MigrateSilentCommand
 
 
-def _set_suffix_to_test_databases(suffix):
+def _set_suffix_to_test_databases(suffix: str) -> None:
     from django.conf import settings
 
     for db_settings in settings.DATABASES.values():
@@ -189,33 +300,24 @@ def _set_suffix_to_test_databases(suffix):
 
 
 @pytest.fixture(scope="function")
-def db(request, django_db_setup, django_db_blocker):
+def db(_django_db_helper: None) -> None:
     """Require a django test database.
 
     This database will be setup with the default fixtures and will have
     the transaction management disabled. At the end of the test the outer
     transaction that wraps the test itself will be rolled back to undo any
     changes to the database (in case the backend supports transactions).
-    This is more limited than the ``transactional_db`` resource but
+    This is more limited than the ``transactional_db`` fixture but
     faster.
 
-    If multiple database fixtures are requested, they take precedence
-    over each other in the following order (the last one wins): ``db``,
-    ``transactional_db``, ``django_db_reset_sequences``.
+    If both ``db`` and ``transactional_db`` are requested,
+    ``transactional_db`` takes precedence.
     """
-    if "django_db_reset_sequences" in request.fixturenames:
-        request.getfixturevalue("django_db_reset_sequences")
-    if (
-        "transactional_db" in request.fixturenames
-        or "live_server" in request.fixturenames
-    ):
-        request.getfixturevalue("transactional_db")
-    else:
-        _django_db_fixture_helper(request, django_db_blocker, transactional=False)
+    # The `_django_db_helper` fixture checks if `db` is requested.
 
 
 @pytest.fixture(scope="function")
-def transactional_db(request, django_db_setup, django_db_blocker):
+def transactional_db(_django_db_helper: None) -> None:
     """Require a django test database with transaction support.
 
     This will re-initialise the django database for each test and is
@@ -224,35 +326,51 @@ def transactional_db(request, django_db_setup, django_db_blocker):
     If you want to use the database with transactions you must request
     this resource.
 
-    If multiple database fixtures are requested, they take precedence
-    over each other in the following order (the last one wins): ``db``,
-    ``transactional_db``, ``django_db_reset_sequences``.
+    If both ``db`` and ``transactional_db`` are requested,
+    ``transactional_db`` takes precedence.
     """
-    if "django_db_reset_sequences" in request.fixturenames:
-        request.getfixturevalue("django_db_reset_sequences")
-    _django_db_fixture_helper(request, django_db_blocker, transactional=True)
+    # The `_django_db_helper` fixture checks if `transactional_db` is requested.
 
 
 @pytest.fixture(scope="function")
-def django_db_reset_sequences(request, django_db_setup, django_db_blocker):
+def django_db_reset_sequences(
+    _django_db_helper: None,
+    transactional_db: None,
+) -> None:
     """Require a transactional test database with sequence reset support.
 
-    This behaves like the ``transactional_db`` fixture, with the addition
-    of enforcing a reset of all auto increment sequences.  If the enquiring
+    This requests the ``transactional_db`` fixture, and additionally
+    enforces a reset of all auto increment sequences.  If the enquiring
     test relies on such values (e.g. ids as primary keys), you should
     request this resource to ensure they are consistent across tests.
-
-    If multiple database fixtures are requested, they take precedence
-    over each other in the following order (the last one wins): ``db``,
-    ``transactional_db``, ``django_db_reset_sequences``.
     """
-    _django_db_fixture_helper(
-        request, django_db_blocker, transactional=True, reset_sequences=True
-    )
+    # The `_django_db_helper` fixture checks if `django_db_reset_sequences`
+    # is requested.
+
+
+@pytest.fixture(scope="function")
+def django_db_serialized_rollback(
+    _django_db_helper: None,
+    db: None,
+) -> None:
+    """Require a test database with serialized rollbacks.
+
+    This requests the ``db`` fixture, and additionally performs rollback
+    emulation - serializes the database contents during setup and restores
+    it during teardown.
+
+    This fixture may be useful for transactional tests, so is usually combined
+    with ``transactional_db``, but can also be useful on databases which do not
+    support transactions.
+
+    Note that this will slow down that test suite by approximately 3x.
+    """
+    # The `_django_db_helper` fixture checks if `django_db_serialized_rollback`
+    # is requested.
 
 
 @pytest.fixture()
-def client():
+def client() -> "django.test.client.Client":
     """A Django test client instance."""
     skip_if_no_django()
 
@@ -262,7 +380,17 @@ def client():
 
 
 @pytest.fixture()
-def django_user_model(db):
+def async_client() -> "django.test.client.AsyncClient":
+    """A Django test async client instance."""
+    skip_if_no_django()
+
+    from django.test.client import AsyncClient
+
+    return AsyncClient()
+
+
+@pytest.fixture()
+def django_user_model(db: None):
     """The class of Django's user model."""
     from django.contrib.auth import get_user_model
 
@@ -270,13 +398,17 @@ def django_user_model(db):
 
 
 @pytest.fixture()
-def django_username_field(django_user_model):
+def django_username_field(django_user_model) -> str:
     """The fieldname for the username used with Django's user model."""
     return django_user_model.USERNAME_FIELD
 
 
 @pytest.fixture()
-def admin_user(db, django_user_model, django_username_field):
+def admin_user(
+    db: None,
+    django_user_model,
+    django_username_field: str,
+):
     """A Django admin user.
 
     This uses an existing user with username "admin", or creates a new one with
@@ -293,18 +425,20 @@ def admin_user(db, django_user_model, django_username_field):
         # so we can assume so as well.
         user = UserModel._default_manager.get_by_natural_key(username)
     except UserModel.DoesNotExist:
-        user = UserModel._default_manager.create_superuser(
-            **{
-                "email": "admin@example.com",
-                "password": "password",
-                username_field: username,
-            }
-        )
+        user_data = {}
+        if "email" in UserModel.REQUIRED_FIELDS:
+            user_data["email"] = "admin@example.com"
+        user_data["password"] = "password"
+        user_data[username_field] = username
+        user = UserModel._default_manager.create_superuser(**user_data)
     return user
 
 
 @pytest.fixture()
-def admin_client(db, admin_user):
+def admin_client(
+    db: None,
+    admin_user,
+) -> "django.test.client.Client":
     """A Django test client logged in as an admin user."""
     from django.test.client import Client
 
@@ -314,7 +448,7 @@ def admin_client(db, admin_user):
 
 
 @pytest.fixture()
-def rf():
+def rf() -> "django.test.client.RequestFactory":
     """RequestFactory instance"""
     skip_if_no_django()
 
@@ -323,10 +457,20 @@ def rf():
     return RequestFactory()
 
 
-class SettingsWrapper:
-    _to_restore = []
+@pytest.fixture()
+def async_rf() -> "django.test.client.AsyncRequestFactory":
+    """AsyncRequestFactory instance"""
+    skip_if_no_django()
 
-    def __delattr__(self, attr):
+    from django.test.client import AsyncRequestFactory
+
+    return AsyncRequestFactory()
+
+
+class SettingsWrapper:
+    _to_restore = []  # type: List[Any]
+
+    def __delattr__(self, attr: str) -> None:
         from django.test import override_settings
 
         override = override_settings()
@@ -337,19 +481,19 @@ class SettingsWrapper:
 
         self._to_restore.append(override)
 
-    def __setattr__(self, attr, value):
+    def __setattr__(self, attr: str, value) -> None:
         from django.test import override_settings
 
         override = override_settings(**{attr: value})
         override.enable()
         self._to_restore.append(override)
 
-    def __getattr__(self, item):
+    def __getattr__(self, attr: str):
         from django.conf import settings
 
-        return getattr(settings, item)
+        return getattr(settings, attr)
 
-    def finalize(self):
+    def finalize(self) -> None:
         for override in reversed(self._to_restore):
             override.disable()
 
@@ -398,7 +542,7 @@ def live_server(request):
 
 
 @pytest.fixture(autouse=True, scope="function")
-def _live_server_helper(request):
+def _live_server_helper(request) -> None:
     """Helper to make live_server work, internal to pytest-django.
 
     This helper will dynamically request the transactional_db fixture
@@ -424,14 +568,22 @@ def _live_server_helper(request):
 
 
 @contextmanager
-def _assert_num_queries(config, num, exact=True, connection=None, info=None):
+def _assert_num_queries(
+    config,
+    num: int,
+    exact: bool = True,
+    connection=None,
+    info=None,
+) -> Generator["django.test.utils.CaptureQueriesContext", None, None]:
     from django.test.utils import CaptureQueriesContext
 
     if connection is None:
-        from django.db import connection
+        from django.db import connection as conn
+    else:
+        conn = connection
 
     verbose = config.getoption("verbose") > 0
-    with CaptureQueriesContext(connection) as context:
+    with CaptureQueriesContext(conn) as context:
         yield context
         num_performed = len(context)
         if exact:
@@ -464,3 +616,38 @@ def django_assert_num_queries(pytestconfig):
 @pytest.fixture(scope="function")
 def django_assert_max_num_queries(pytestconfig):
     return partial(_assert_num_queries, pytestconfig, exact=False)
+
+
+@contextmanager
+def _capture_on_commit_callbacks(
+    *,
+    using: Optional[str] = None,
+    execute: bool = False
+):
+    from django.db import DEFAULT_DB_ALIAS, connections
+    from django.test import TestCase
+
+    if using is None:
+        using = DEFAULT_DB_ALIAS
+
+    # Polyfill of Django code as of Django 3.2.
+    if get_django_version() < (3, 2):
+        callbacks = []  # type: List[Callable[[], Any]]
+        start_count = len(connections[using].run_on_commit)
+        try:
+            yield callbacks
+        finally:
+            run_on_commit = connections[using].run_on_commit[start_count:]
+            callbacks[:] = [func for sids, func in run_on_commit]
+            if execute:
+                for callback in callbacks:
+                    callback()
+
+    else:
+        with TestCase.captureOnCommitCallbacks(using=using, execute=execute) as callbacks:
+            yield callbacks
+
+
+@pytest.fixture(scope="function")
+def django_capture_on_commit_callbacks():
+    return _capture_on_commit_callbacks
